//shading:         phong
//lighting model:  blinn
//lighting type:   point

// --------------------------------------------------------------------------------------------------
// PARAMETERS:
// --------------------------------------------------------------------------------------------------

//transforms
float4x4 tW: WORLD;        //the models world matrix
float4x4 tV: VIEW;         //view matrix as set via Renderer (DX9)
float4x4 tWV: WORLDVIEW;
float4x4 tWVP: WORLDVIEWPROJECTION;
float4x4 tP: PROJECTION;   //projection matrix as set via Renderer (DX9)


//light properties
float3 lPos <string uiname="Light Position";> = {0, 5, -2};       //light position in world space
float lAtt0 <String uiname="Light Attenuation 0"; float uimin=0.0;> = 0;
float lAtt1 <String uiname="Light Attenuation 1"; float uimin=0.0;> = 0.3;
float lAtt2 <String uiname="Light Attenuation 2"; float uimin=0.0;> = 0;
float4 lAmb  : COLOR <String uiname="Ambient Color";>  = {0.15, 0.15, 0.15, 1};
float4 lDiff : COLOR <String uiname="Diffuse Color";>  = {0.85, 0.85, 0.85, 1};
float4 lSpec : COLOR <String uiname="Specular Color";> = {0.35, 0.35, 0.35, 1};
float lPower <String uiname="Power"; float uimin=0.0;> = 25.0;     //shininess of specular highlight
float lRange <String uiname="Light Range"; float uimin=0.0;> = 10.0;


//       _________________________________________________________________________________
//       <   TEXTURES    >
//       

//          _____
//       __/  1  \________________________________________________________________________
//       
         texture Tex1  <string uiname="Texture";>;
         sampler Samp1 = sampler_state    //sampler for doing the texture-lookup
         {
         Texture   = (Tex1);          //apply a texture to the sampler
         MipFilter = LINEAR;          //  LINEAR;   POINT;   NONE;   ANISOTROPIC;  PYRAMIDALQUAD;   GAUSSIANQUAD;    CONVOLUTIONMONO;  FORCE_DWORD;
         MinFilter = LINEAR;          //  LINEAR;   POINT;   NONE;   ANISOTROPIC;  PYRAMIDALQUAD;   GAUSSIANQUAD;    CONVOLUTIONMONO;  FORCE_DWORD;
         MagFilter = LINEAR;          //  LINEAR;   POINT;   NONE;   ANISOTROPIC;  PYRAMIDALQUAD;   GAUSSIANQUAD;    CONVOLUTIONMONO;  FORCE_DWORD;
         //SRGBTEXTURE = 1;
         AddressU = WRAP;          //    CLAMP;   WRAP;   MIRROR;   BORDER;  MIRRORONCE;  FORCE_DWORD;
         AddressV = WRAP;          //    CLAMP;   WRAP;   MIRROR;   BORDER;  MIRRORONCE;  FORCE_DWORD;
         AddressW = WRAP;          //    CLAMP;   WRAP;   MIRROR;   BORDER;  MIRRORONCE;  FORCE_DWORD;
         //BorderColor  = {0.15, 0.15, 0.45, 1};
         };

         float4x4 Tex1Transform: TEXTUREMATRIX <string uiname="Texture Transform";>;
//       
//                 _____
//       _________/  2  \_________________________________________________________________
//       
         texture Tex2  <string uiname="Texture 2";>;
         sampler Samp2 = sampler_state    //sampler for doing the texture-lookup
         {
         Texture   = (Tex2);          //apply a texture to the sampler
         MipFilter = LINEAR;          //  LINEAR;   POINT;   NONE;   ANISOTROPIC;  PYRAMIDALQUAD;   GAUSSIANQUAD;    CONVOLUTIONMONO;  FORCE_DWORD;
         MinFilter = LINEAR;          //  LINEAR;   POINT;   NONE;   ANISOTROPIC;  PYRAMIDALQUAD;   GAUSSIANQUAD;    CONVOLUTIONMONO;  FORCE_DWORD;
         MagFilter = LINEAR;          //  LINEAR;   POINT;   NONE;   ANISOTROPIC;  PYRAMIDALQUAD;   GAUSSIANQUAD;    CONVOLUTIONMONO;  FORCE_DWORD;
         //SRGBTEXTURE = 1;
         AddressU = WRAP;          //    CLAMP;   WRAP;   MIRROR;   BORDER;  MIRRORONCE;  FORCE_DWORD;
         AddressV = WRAP;          //    CLAMP;   WRAP;   MIRROR;   BORDER;  MIRRORONCE;  FORCE_DWORD;
         AddressW = WRAP;          //    CLAMP;   WRAP;   MIRROR;   BORDER;  MIRRORONCE;  FORCE_DWORD;
         //BorderColor  = {0.15, 0.15, 0.45, 1};
         };
         float4x4 Tex2Transform: TEXTUREMATRIX <string uiname="Texture 2 Transform";>;
//       
//                        _____
//       ________________/  3  \___________________________________________________________
//       
         texture  CTRLTex     <string uiname="Control Texture";>;
         sampler  CTRLSamp = sampler_state    //sampler for doing the texture-lookup
         {
         Texture   = (CTRLTex);          //apply a texture to the sampler
         MipFilter = LINEAR;          //  LINEAR;   POINT;   NONE;   ANISOTROPIC;  PYRAMIDALQUAD;   GAUSSIANQUAD;    CONVOLUTIONMONO;  FORCE_DWORD;
         MinFilter = LINEAR;          //  LINEAR;   POINT;   NONE;   ANISOTROPIC;  PYRAMIDALQUAD;   GAUSSIANQUAD;    CONVOLUTIONMONO;  FORCE_DWORD;
         MagFilter = LINEAR;          //  LINEAR;   POINT;   NONE;   ANISOTROPIC;  PYRAMIDALQUAD;   GAUSSIANQUAD;    CONVOLUTIONMONO;  FORCE_DWORD;
         //SRGBTEXTURE = 1;
         AddressU = WRAP;          //    CLAMP;   WRAP;   MIRROR;   BORDER;  MIRRORONCE;  FORCE_DWORD;
         AddressV = WRAP;          //    CLAMP;   WRAP;   MIRROR;   BORDER;  MIRRORONCE;  FORCE_DWORD;
         AddressW = WRAP;          //    CLAMP;   WRAP;   MIRROR;   BORDER;  MIRRORONCE;  FORCE_DWORD;
         //BorderColor  = {0.15, 0.15, 0.45, 1};
         };
         float4x4 CTRLTexTransform: TEXTUREMATRIX <string uiname="Control Texture Transform";>;
         bool     InvCTRLTex                      <string uiname="Invert Control Texture";> = 0;
         bool    UseTex2                         <string uiname="Enable Texture 2";> = 1;
//       
//       

float Alpha < float UIMin = 0.0;  float UIMax = 1.0;> = 1;
float4x4 ColorTransform           <string uiname="Texture Color Transform";>;


//phong point function
float4 PhongPoint(float3 PosW, float3 NormV, float3 ViewDirV, float3 LightDirV)
{

    float d = distance(PosW, lPos);
    float atten = 0;

    //compute attenuation only if vertex within lightrange
    if (d<lRange)
    {
       atten = 1/(saturate(lAtt0) + saturate(lAtt1) * d + saturate(lAtt2) * pow(d, 2));
    }

    float4 amb = lAmb * atten;
    amb.a = 1;

    //halfvector
    float3 H = normalize(ViewDirV + LightDirV);

    //compute blinn lighting
    float4 shades = lit(dot(NormV, LightDirV), dot(NormV, H), lPower);

    float4 diff = lDiff * shades.y * atten;
    diff.a = 1;

    //reflection vector (view space)
    float3 R = normalize(2 * dot(NormV, LightDirV) * NormV - LightDirV);

    //normalized view direction (view space)
    float3 V = normalize(ViewDirV);

    //calculate specular light
    float4 spec = pow(max(dot(R, V),0), lPower*.2) * lSpec;

    return (amb + diff) + spec;
}


struct vs2ps
{
    float4 PosWVP: POSITION;
    float4 TexCd : TEXCOORD0;
    float3 LightDirV: TEXCOORD1;
    float3 NormV: TEXCOORD2;
    float3 ViewDirV: TEXCOORD3;
    float3 PosW: TEXCOORD4;
};

// --------------------------------------------------------------------------------------------------
// VERTEXSHADERS
// --------------------------------------------------------------------------------------------------

vs2ps VS(
    float4 PosO: POSITION,
    float3 NormO: NORMAL,
    float4 TexCd : TEXCOORD0)
{
    //inititalize all fields of output struct with 0
    vs2ps Out = (vs2ps)0;

    Out.PosW = mul(PosO, tW);

    //inverse light direction in view space
    float3 LightDirW = normalize(lPos - Out.PosW);
    Out.LightDirV = mul(LightDirW, tV);
    
    //normal in view space
    Out.NormV = normalize(mul(NormO, tWV));

    //position (projected)
    Out.PosWVP  = mul(PosO, tWVP);
    Out.TexCd = TexCd;
    Out.ViewDirV = -normalize(mul(PosO, tWV));
    return Out;
}

// --------------------------------------------------------------------------------------------------
// PIXELSHADERS:
// --------------------------------------------------------------------------------------------------

float4 PS(vs2ps In): COLOR
{
        float4 map1  = tex2D (    Samp1 , mul ( In.TexCd ,    Tex1Transform )  );        /////////////////////////////////////////////////////////////////
    float4 map2  = tex2D (    Samp2 , mul ( In.TexCd ,    Tex2Transform )  );        /////////////////////////////////////////////////////////////////
           map2 *=  UseTex2;                                                       /////////////////////////////////////////////////////////////////
    float4 swtch   = tex2D ( CTRLSamp , mul ( In.TexCd , CTRLTexTransform ) );
          swtch.a = max ( swtch.r , max ( swtch.g , swtch.b  ) );
            swtch   = lerp  ( InvCTRLTex , 1-InvCTRLTex, swtch )  ;
           
           
           
           
    float4 result      = swtch * map2 +   (1-swtch) * map1;
           result.rgb *= PhongPoint(In.PosW, In.NormV, In.ViewDirV, In.LightDirV);
           result.a *=  Alpha ;
           result  = mul ( result, ColorTransform );
    return result;
}

    //col.rgb *= PhongPoint(In.PosW, In.NormV, In.ViewDirV, In.LightDirV);
// --------------------------------------------------------------------------------------------------
// TECHNIQUES:
// --------------------------------------------------------------------------------------------------

technique TPhongPoint
{
    pass P0
    {
        //Wrap0 = U;  // useful when mesh is round like a sphere
        VertexShader = compile vs_1_0 VS();
        PixelShader = compile ps_2_0 PS();
    }
}
